@@ -1,6 +1,4 @@
 /*
- * $Xorg: gethost.c,v 1.5 2001/02/09 02:05:38 xorgcvs Exp $
- * $XdotOrg: $
  * 
 Copyright 1989, 1998  The Open Group
 
@@ -27,8 +25,6 @@ in this Software without prior written authorization from The Open Group.
  * Author:  Jim Fulton, MIT X Consortium
  */
 
-/* $XFree86: xc/programs/xauth/gethost.c,v 3.20 2003/07/27 12:34:25 herrb Exp $ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -280,7 +276,28 @@ struct addrlist *get_address_info (
 		src = buf;
 		len = strlen (buf);
 	    }
-	} else {
+	} else if(prefix == 0 && (strncmp (fulldpyname, "/tmp/launch", 11) == 0)) {
+        /* Use the bundle id (part preceding : in the basename) as our src id */
+        char *c;
+#ifdef HAVE_STRLCPY
+        strlcpy(buf, strrchr(fulldpyname, '/') + 1, sizeof(buf));
+#else
+        strncpy(buf, strrchr(fulldpyname, '/') + 1, sizeof(buf));
+	buf[sizeof(buf) - 1] = '\0';
+#endif
+
+        c = strchr(buf, ':');
+        
+        /* In the legacy case with no bundle id, use the full path */
+        if(c == buf) {
+            src = fulldpyname;
+        } else {
+            *c = '\0';
+            src = buf;
+        }
+
+        len = strlen(src);
+    } else {
 	    src = fulldpyname;
 	    len = prefix;
 	}
